from trackc.pa.cmaps import (
    colorC,
    fruitpunch,
    fruitpunch2
)

from trackc.pa.palettes import (
    trackcl_9,
    trackcl_11,
    vega_10_trackc,
    default_20,
    zeileis_28,
    default_102
)
