.wy-nav-top {
    background-color: #FF8C38 !important;
}
.wy-side-nav-search {
    background-color: #FF8C38 !important;
}


div[class^="highlight"] a {
    background-color: #E6E6E6;
}

div[class^="highlight"] a:hover {
    background-color: #ABECFC;
}

.rst-versions {
    position: relative;
}
.rst-versions.shift-up {
    overflow-y: visible;
}

a[class^="sphx-glr-backref-module-"] {
    text-decoration: none;
    background-color: rgba(0, 0, 0, 0) !important;
}
a.sphx-glr-backref-module-sphinx_gallery {
    text-decoration: underline;
    background-color: #E6E6E6;
}

.anim-state label {
    display: inline-block;
}
